const { Controller } = require('vertex360')({ site_id: process.env.TURBO_APP_ID })
const Base = require('../models/Base')

class BaseController extends Controller {
  constructor () {
    super(Base, process.env)
  }

  async get (params) {
    const bases = await Base.find(params, Controller.parseFilters(params))
    return Base.convertToJson(bases)
  }

  async getById (id) {
    const base = await Base.findById(id)
    if (base == null) {
      throw new Error(`${Base.resourceName} ${id} not found.`)
    }

    return base.summary()
  }

  async post (body) {
    const base = await Base.create(body)
    return base.summary()
  }

  async put (id, params) {
    const base = await Base.findByIdAndUpdate(id, params, { new: true })
    return base.summary()
  }

  async delete (id) {
    const base = await Base.findByIdAndRemove(id)
    return base
  }
}

module.exports = new BaseController()
